require(openxlsx)
require(betareg)
require(xtable)

# path and file settings
path  <- "~/Publications/_InProgress/TC-2017-Rupesh/data/reprocessing"
file1 <- "Result_OneRowPerSample.csv"
file2 <- "NRAL Stephan Gruber Grain Size Dec 16, 2019.xlsx"
file3 <- "TerrainTypes.xlsx"
file4 <- "NGO-15006-NT_SummerDD_Database_VisibleIce.xlsx"

# read, merge and tidy up
data1 <- read.csv(file.path(path, file1), 1)
data2 <- read.xlsx(file.path(path, file2), sheet = 1)
data3 <- read.xlsx(file.path(path, file3), sheet = 1)
data4 <- read.xlsx(file.path(path, file4), sheet = "tblDDLogs") #IceVE_pct
data  <- merge(data1, data2, by="SampleID", all.x=TRUE)
data  <- subset(data, select=-c(X))
names(data)[2:3] <- c("FromDepth","ToDepth")

# borehole ID
data$BoreholeID <- substr(as.character(data$SampleID),1,13)
data  <- merge(data, data3, by="BoreholeID")

# compute logged withs and drop, correct one "to" value: correction is obvious from
# sheet tblTCR 
data4$To[(data4$HoleID == "NGO-DD15-2033") & (data4$From == 4.5)] <- 5.1

# visible ice summary statistics
mask <- (!is.na(data4$IceVE_pct)) & (data4$Frozen == 'Yes')
VI_val <- sum(data4$IceVE_pct[mask] * (data4$To[mask] - data4$From[mask])) / 
          sum((data4$To[mask] - data4$From[mask]))
# in this analysis, no logged VI is interpreted as 0% VI           
data4$IceVE_pct[!mask] <- 0          
VI_all <- sum(data4$IceVE_pct[data4$Frozen == 'Yes'] * (data4$To[data4$Frozen == 'Yes'] - 
                                                        data4$From[data4$Frozen == 'Yes']), 
                                                        na.rm=TRUE) / 
          sum((data4$To[data4$Frozen == 'Yes'] - data4$From[data4$Frozen == 'Yes']), na.rm=TRUE)
print(paste("Visible ice estimated percent - Logged values:", round(VI_val,0), 
            "%, NA interpreted as 0:", round(VI_all,0),"%"))
print("=======================================================")


# aggregate visible ice percent estimates from core sections to samples
data$IceVE_pct <- NA
for (l in 1:nrow(data)) { # loop over all samples
	# mask to extract overlapping intervals
	mask <- (data4$From < data$ToDepth[l]) & 
	        (data4$To   > data$FromDepth[l]) & 
	        (data4$HoleID == data$BoreholeID[l]) & 
			(data4$Frozen == 'Yes')
			
	# restrict intervals to intersection of logged and sample		
    tto <- ifelse(data4$To[mask]  <  data$ToDepth[l],   data4$To[mask],  data$ToDepth[l])
    tfr <- ifelse(data4$From[mask] > data$FromDepth[l], data4$From[mask], data$FromDepth[l])
    data$IceVE_pct[l] <- round(sum(data4$IceVE_pct[mask] * (tto-tfr), na.rm=TRUE) / sum(tto-tfr, na.rm=TRUE),0)

}


# fix data types
data$Gravel_perc <- as.numeric(data$Gravel_perc)
data$Sand_perc   <- as.numeric(data$Sand_perc)
data$Fines_perc  <- as.numeric(data$Fines_perc)

# volumetric percentage of coarse fragments ($V_c$) relative to the total sample volume
data$coarse_totalsample_perc <- data$Gravel_perc * (1-data$ExIceCont_perc/100)

# SOM_perc: insert 80.0 where NA
data$SOM_perc <- as.numeric(data$SOM_perc)
data$SOM_orig <- "observation"
data$SOM_orig[is.na(data$SOM_perc)] <- "estimate"
data$SOM_perc[is.na(data$SOM_perc)] <- 80.0

# estimate missing value (should not be set to 80%) by interpolation from above and below
data$SOM_perc[data$SampleID=="NGO-DD15-2011-13"] <- (data$SOM_perc[data$SampleID=="NGO-DD15-2011-12"] +
                                                     data$SOM_perc[data$SampleID=="NGO-DD15-2011-14"])/2


# soil pits were dug only in thawed soil
mask <- data$Source != "Borehole"
data$Frozen[mask] <- "No"


# =============================================================
# FIX QUANTITIES FOUND IN RUPESH'S SCRIPTS
# =============================================================

# Terrain type: Organics [see NGO-DD15-1005-13 in NGO-15006-NT_SummerDD_Database_VisibleIce.xlsx]
data$FromDepth[data$SampleID=="NGO-DD15-1005-12"]   <- 1.27
data$ToDepth[data$SampleID=="NGO-DD15-1005-12"]     <- 1.36
data$SampleDepth[data$SampleID=="NGO-DD15-1005-12"] <- 1.31
data$UnitCode[data$SampleID=="NGO-DD15-1005-12"]    <- "Ice"
				
# Terrain type: Till [see log for NGO-DD15-2004 in NGO-15006-NT_SummerDD_Database_VisibleIce.xlsx]					
data$FromDepth[data$SampleID=="NGO-DD15-2004-06"]   <- 3.5
data$ToDepth[data$SampleID=="NGO-DD15-2004-06"]     <- 3.9
data$SampleDepth[data$SampleID=="NGO-DD15-2004-06"] <- 3.7
data$UnitCode[data$SampleID=="NGO-DD15-2004-06"]    <- 'Gravel/Cobbles'
data$GroIceDes[data$SampleID=="NGO-DD15-2004-06"]   <- 'ICE+'			


# =============================================================


# =============================================================
# fit beta regression
data$fine_nogravel_perc <- data$NRAL_fine_perc / (data$NRAL_fine_perc + data$NRAL_medium_perc)
gm2 <- betareg(fine_nogravel_perc ~ Sand_perc + Fines_perc + SOM_perc + GWCWetBase_perc, data)
summary(gm2)
print(paste("Standard devision of residuals:",round(sd(gm2$residuals),2)))
pred <- data.frame(betareg=predict(gm2, data))

# tally
print(paste("total rows          :", nrow(data)))
print(paste("NRAL rows           :", sum(!is.na(data$fine_nogravel_perc))))
print(paste("Beta regression rows:", sum( is.na(data$fine_nogravel_perc))))

# combine measured and estimated
data$fine_nogravel_orig <- "NRAL lab measurement"
data$fine_nogravel_orig[is.na(data$fine_nogravel_perc)] <- "Beta regression on NRAL"
data$fine_nogravel_perc[is.na(data$fine_nogravel_perc)] <- pred$betareg[is.na(data$fine_nogravel_perc)]
data$fine_nogravel_perc <- data$fine_nogravel_perc*100 #scale back to percent
print("=======================================================")
# =============================================================



# =============================================================
# Soil organic-carbon concentration (SOCc, % mass) 
# =============================================================
data$SOC_perc <- data$SOM_perc / 2.13

# Dry bulk density of fines (DBD, kgm−3) Hossain et al., 2015
data$DryBulkDensity_kgm3 <- 71 + 1322 * exp(-0.071 * data$SOC_perc)

# SOCd soil organic-carbon densoty [kg C m-3]
data$SOCd <- data$SOC_perc/100 * data$fine_nogravel_perc/100 * 
             data$DryBulkDensity_kgm3 * 
             (1-(data$ExIceCont_perc+data$coarse_totalsample_perc)/100)



# =============================================================
# high-carbon scenario to estimate uncertainty
p05_high <- data$fine_nogravel_perc/100 + 0.1
p05_high[p05_high > 1] <- 1

Vc_high <- data$coarse_totalsample_perc - 10
Vc_high[Vc_high < 0] <- 0

DBD_high <- data$DryBulkDensity_kgm3 + 50

data$SOCd_high_scenario <- data$SOC_perc/100 * p05_high * DBD_high * 
             (1-(data$ExIceCont_perc+Vc_high)/100)
             
             
# low-carbon scenario to estimate uncertainty
p05_low <- data$fine_nogravel_perc/100 - 0.1
p05_low[p05_low < 0] <- 0

Vc_low <- data$coarse_totalsample_perc + 10
Vc_low[Vc_low > 100] <- 100

DBD_low <- data$DryBulkDensity_kgm3 - 50

data$SOCd_low_scenario <- data$SOC_perc/100 * p05_low * DBD_low * 
             (1-(data$ExIceCont_perc+Vc_low)/100)

# feedback on relative change of SOCd             
print(paste("Scenario values are", 
            round(1-mean(data$SOCd_low_scenario / data$SOCd),2)*100, 
            "to", 
            round(mean(data$SOCd_high_scenario / data$SOCd)-1,2)*100,
            "% of best estimate for SOCd."))                
print("=======================================================")
# =============================================================
# OUTPUT
# =============================================================             

# make pretty
round0 <- c("Gravel_perc", "Sand_perc", "Fines_perc", "IceVE_pct")
round1 <- c("DryBulkDensity_kgm3")
round2 <- c("FromDepth", "ToDepth", "SampleDepth", "GWCWetBase_perc", "ExIceCont_perc", 
            "GWCDryBase_perc", "SOM_perc", "Cond_µSpercm", "Cond_Std", "NRAL_coarse_perc",             
            "NRAL_medium_perc", "NRAL_fine_perc","coarse_totalsample_perc", "fine_nogravel_perc", 
            "SOC_perc", "SOCd" , "SOCd_high_scenario", "SOCd_low_scenario")
round3 <- c("CorrectionFactor")
round4 <- c("Cond_mSpercm","Calcium_mgl", "Calcium_meql", "Calcium_meq100g", "Calcium_meq100gWetSoil",       
            "Magnesium_mgl" , "Magnesium_meql", "Magnesium_meq100g", "Magnesium_meq100gWetSoil",     
            "Potassium_mgl", "Potassium_meql", "Potassium_meq100g", "Potassium_meq100gWetSoil",     
            "Sodium_mgl", "Sodium_meql", "Sodium_meq100g", "Sodium_meq100gWetSoil", "SolubleCations_mgl",           
            "SolubleCations_meql", "SolubleCations_meq100g", "SolubleCations_meq100gWetSoil")

data[, round0] <- round(data[, round0],0)
data[, round1] <- round(data[, round1],1)
data[, round2] <- round(data[, round2],2)
data[, round3] <- round(data[, round3],3)
data[, round4] <- round(data[, round4],4)
data <- subset(data, select=-c(WaterMassper100g, WaterMassper100gWetSoil))

# write output data
write.csv(data, file.path(path, "Lac_de_Gras_permafrost_20200126.csv"), 
          row.names=FALSE, na = "")


# =============================================================
# Solutes 
# =============================================================             

# get samples that are logged as frozen or unfrozen
lfu  <- na.omit(subset(data, (Frozen == "Yes") | (Frozen == "No"), 
                       select=c(SOM_perc,TerrainType,Frozen,TerrainType,
                                SolubleCations_meq100g, 
                                SolubleCations_meq100gWetSoil)))
org  <- lfu$SOM_perc > 30  # page 97 (17, 79) of The Canadian System of Soil Classification 1998
fro  <- lfu$Frozen == "Yes"

print(paste(nrow(lfu), "samples with frozen/unfrozen data"))         
print("Median dry, median wet, n")    
print(paste(round(median(lfu$SolubleCations_meq100g[org & fro]),2), 
            round(median(lfu$SolubleCations_meq100gWetSoil[org & fro]),2),  
            nrow(lfu[org & fro,]),   "Organic: frozen"))
print(paste(round(median(lfu$SolubleCations_meq100g[org & !fro]),2), 
            round(median(lfu$SolubleCations_meq100gWetSoil[org & !fro]),2), 
            nrow(lfu[org & !fro,]),  "Organic: thawed"))
print(paste(round(median(lfu$SolubleCations_meq100g[!org & fro]),2), 
            round(median(lfu$SolubleCations_meq100gWetSoil[!org & fro]),2), 
            nrow(lfu[!org & fro,]),   "Mineral: frozen"))
print(paste(round(median(lfu$SolubleCations_meq100g[!org & !fro]),2), 
            round(median(lfu$SolubleCations_meq100gWetSoil[!org & !fro]),2), 
            nrow(lfu[!org & !fro,]),  "Mineral: thawed"))          
             

kw <- subset(lfu, select=c(SolubleCations_meq100g, SolubleCations_meq100gWetSoil))
kw$group              <- "organic-frozen"
kw$group[org & !fro]  <- "organic-thawed"
kw$group[!org & fro]  <- "mineral-frozen"
kw$group[!org & !fro] <- "mineral-thawed"

print(kruskal.test(SolubleCations_meq100g ~ group, data = kw[grepl("thaw", kw$group),]))
print(kruskal.test(SolubleCations_meq100g ~ group, data = kw[grepl("froz", kw$group),]))       
print(kruskal.test(SolubleCations_meq100g ~ group, data = kw[grepl("orga", kw$group),]))             
print(kruskal.test(SolubleCations_meq100g ~ group, data = kw[grepl("mine", kw$group),]))

print(kruskal.test(SolubleCations_meq100gWetSoil ~ group, data = kw[grepl("thaw", kw$group),]))
print(kruskal.test(SolubleCations_meq100gWetSoil ~ group, data = kw[grepl("froz", kw$group),]))       
print(kruskal.test(SolubleCations_meq100gWetSoil ~ group, data = kw[grepl("orga", kw$group),]))             
print(kruskal.test(SolubleCations_meq100gWetSoil ~ group, data = kw[grepl("mine", kw$group),]))

print("Median total cation concentrations for mineral soils")                 
print(paste(round(median(lfu$SolubleCations_meq100g[!org & fro & lfu$TerrainType=='Till']),2), "Till frozen"))                 
print(paste(round(median(lfu$SolubleCations_meq100g[!org & !fro & lfu$TerrainType=='Till']),2),"Till thawed"))                   
print(paste(round(median(lfu$SolubleCations_meq100g[!org & fro & lfu$TerrainType=='Valley']),2), "Valley frozen"))                 
print(paste(round(median(lfu$SolubleCations_meq100g[!org & !fro & lfu$TerrainType=='Valley']),2),"Valley thawed"))  
print(paste(round(median(lfu$SolubleCations_meq100g[!org & fro & lfu$TerrainType=='Eskers']),2), "Eskers frozen"))                 
print(paste(round(median(lfu$SolubleCations_meq100g[!org & !fro & lfu$TerrainType=='Eskers']),2),"Eskers thawed"))  

print("Mean total cation concentrations for mineral soils")                 
print(paste(round(mean(lfu$SolubleCations_meq100g[!org & fro & lfu$TerrainType=='Till']),2), "Till frozen"))                 
print(paste(round(mean(lfu$SolubleCations_meq100g[!org & !fro & lfu$TerrainType=='Till']),2),"Till thawed"))                   
print(paste(round(mean(lfu$SolubleCations_meq100g[!org & fro & lfu$TerrainType=='Valley']),2), "Valley frozen"))                 
print(paste(round(mean(lfu$SolubleCations_meq100g[!org & !fro & lfu$TerrainType=='Valley']),2),"Valley thawed"))  
print(paste(round(mean(lfu$SolubleCations_meq100g[!org & fro & lfu$TerrainType=='Eskers']),2), "Eskers frozen"))                 
print(paste(round(mean(lfu$SolubleCations_meq100g[!org & !fro & lfu$TerrainType=='Eskers']),2),"Eskers thawed"))  
print("=======================================================")

# =============================================================
# CARBON UPSCALING
# =============================================================  
esker   <- 11 * 1/100  # to 1m
organic <- 29 * 6/100  # to 1m
Till1   <- 13 * 21/100 # to 1m
Till2   <- 13 * 24/100 # to 1m
Till3   <- 13 *  3/100 # to 1m
print("Area-averages soil organic-carbon storage 0-1m (kg C m−2):")
print(round(esker+organic+Till1+Till2+Till3),0)

esker   <- 17 * 1/100  # to 3m
organic <- 67 * 6/100  # to 3m
Till1   <- 13 * 21/100 # to 1m
Till2   <- 27 * 24/100 # to 3m
Till3   <- 27 *  3/100 # to 3m
print("Area-averages soil organic-carbon storage 0-3m (kg C m−2):")
print(round(esker+organic+Till1+Till2+Till3),0)

#esker   <- 34 * 1/100  # to 10m
#organic <- 89 * 6/100  # to 5m
#Till1   <- 13 * 21/100 # to 1m
#Till2   <- (33+1.3)   * 24/100 # to 6m
#Till3   <- (40+8*1.3) *  3/100 # to 18m
#print("Area-averages soil organic-carbon storage total (kg C m−2):")
#print(round(esker+organic+Till1+Till2+Till3),0)

print("=======================================================")      


# === statisctics per terrain type: write LaTex table

# helper function
makerow <- function(tmp, depthstring, thickness) {
	row <- NULL
	row <- cbind(row, depthstring)
	for (tt in c("Organics","Valley","Till","Eskers")) {
		row <- cbind(row, paste0(
	                  round(mean(tmp$SOCd[tmp$TerrainType == tt],  na.rm=TRUE),1), ' (', 
	                  round( min(tmp$SOCd[tmp$TerrainType == tt],  na.rm=TRUE)),   '--',
	                  round( max(tmp$SOCd[tmp$TerrainType == tt],  na.rm=TRUE)),   ') [',
	                  length(    tmp$SOCd[tmp$TerrainType == tt]), ']'),
	                  round(mean(tmp$SOCd[tmp$TerrainType == tt] * thickness, na.rm=TRUE),1)
	                  )
	}
	row <- data.frame(row)
	return(row)
}


#settings
maskcol <- c('TerrainType', 'SOCd')
res <- NULL

maskrow <- (data$SampleDepth >= 0) & (data$SampleDepth < 0.3)
tmp <- data[maskrow, maskcol]
res <- rbind(res, makerow(tmp, "0--0.3 m", 0.3))

maskrow <- (data$SampleDepth >= 0.3) & (data$SampleDepth < 1.0)
tmp <- data[maskrow, maskcol]
res <- rbind(res, makerow(tmp, "0.3--1 m", 0.7))

maskrow <- (data$SampleDepth >= 1.0) & (data$SampleDepth < 2.0)
tmp <- data[maskrow, maskcol]
res <- rbind(res, makerow(tmp, "1--2 m", 1))

maskrow <- (data$SampleDepth >= 2.0) & (data$SampleDepth < 3.0)
tmp <- data[maskrow, maskcol]
res <- rbind(res, makerow(tmp, "2--3 m", 1))

maskrow <- (data$SampleDepth >= 3.0) & (data$SampleDepth < 5.0)
tmp <- data[maskrow, maskcol]
res <- rbind(res, makerow(tmp, "3--5 m", 2))

maskrow <- (data$SampleDepth >= 5.0) & (data$SampleDepth < 10.0)
tmp <- data[maskrow, maskcol]
res <- rbind(res, makerow(tmp, "5--10 m", 5))

# assign proper names
names(res) <- c("Depth", "Organics","OrganicsC","Valley","ValleyC","Till","TillC","Eskers","EskersC")

# compute cumulative sums
res$OrganicsC <- as.character(round(cumsum(as.numeric(as.character(res$OrganicsC))),0))
res$ValleyC   <- as.character(round(cumsum(as.numeric(as.character(res$ValleyC))),0))
res$TillC     <- as.character(round(cumsum(as.numeric(as.character(res$TillC))),0))
res$EskersC   <- as.character(round(cumsum(as.numeric(as.character(res$EskersC))),0))
print("=== TABLE for inclusion in LaTex =============================")
print(xtable(res), include.rownames=FALSE)
print("=== TABLE ====================================================")


# ============================================================= 
# ============================================================= 
# run scripts for preparing plots per terrain type 			
source(file.path(path, "Combined_Plots.R")) 


